#!/bin/sh
# ZipitZ2 u-boot flash script
# Copyright (C) 2010 Marek Vasut <marek.vasut@gmail.com>
# This is under GPL v3
# Modified by mozzwald http://www.mozzwald.com
# Modified by rkdavis  http://russelldavis.org

# filename variables
helpfile=/mnt/mmcblk0p1/flashdumps/help.txt
image=/tmp/image.img
partlist=/tmp/plist.lst
flashdir=/mnt/mmcblk0p1/flashdumps
flashdefs=flash.def
restoredir="`date +%m%d%Y_%H%M%S`"
changelog=change.log

# Versions and Dialog settings
SCRIPTNAME="Z2uFlasher"
SCRIPTVER="v0.3"
SCRIPTSUBNAME="u-boot"
DIALOGRC=/mnt/mmcblk0p1/etc/.dialogrc
DIALOGSIZE="12 38"
BACKTITLE="$SCRIPTNAME $SCRIPTVER  |  $SCRIPTSUBNAME"
#BEGIN="--begin 2 1"

# Help Function
help(){
    BACKTITLE="$SCRIPTNAME $SCRIPTVER  |  Help"
    # View the changelog if found
    if [ -f "$helpfile" ]; then
        dialog $BEGIN --backtitle "$BACKTITLE" --textbox "$helpfile" 15 44
    else
        dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "Help file not found, so sorry!" $DIALOGSIZE
    fi
    return 0
}

# Backup Function
backup() {
    dialog $BEGIN --backtitle "$BACKTITLE" --inputbox "Enter a name for the backup directory" $DIALOGSIZE "$restoredir" 2>/tmp/restoredir
    if [ $? -eq 255 ]; then
        return 255
    fi
    backupname=`cat /tmp/restoredir | sed 's/\ /_/g'`
    if [ -z "$backupname" ]; then
        backupname=`date +%m%d%Y_%H%M%S`
    fi
    restoredir="$flashdir/$backupname"
    BACKTITLE="$SCRIPTNAME $SCRIPTVER  |  Backup: $backupname"
    mkdir "$restoredir"
    cd "$restoredir"
    
    # fix mtdblock partitions, create partition list and backup old flash
    flashsz=0
    while read line; do
            i=`echo $line | grep mtdblock | cut -d " " -f 4`
            if [ -n "$i" ]; then
                    # fix the mtdblock
                    #echo "Fixing /dev/$i"
                    blocknum=`echo "$i" | sed -e "s/^.*\(.\)$/\1/"`
                    if [ -e "/dev/$i" ]; then
                            rm "/dev/$i"
                    fi
                    mknod "/dev/$i" b 31 "$blocknum"
                    # write the partition list
                    partsz=`wc -c "/dev/$i" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
                    #echo "/dev/$i $partsz" >> "$partlist"
                    # Backup Partitions & Create flash.def
                    (/mnt/mmcblk0p1/bin/pv-u -n "/dev/$i" | dd of="$restoredir/$i.bak") 2>&1 | dialog $BEGIN --backtitle "$BACKTITLE" --gauge "Backing up /dev/$i" $DIALOGSIZE 0
                    #dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Backing up /dev/$i" $DIALOGSIZE
                    #dd if="/dev/$i" of="$restoredir/$i.bak"
                    mtdmd5=`md5sum "$restoredir/$i.bak" | cut -d " " -f 1`
                    echo "$i.bak $flashsz $mtdmd5" >> "$restoredir"/flash.def
                    flashsz=`expr "$flashsz" + "$partsz"`
                    mtdcount=`expr "$blocknum" + 1`
            fi
    done </proc/partitions
    
    dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "Backup complete and saved to $restoredir" $DIALOGSIZE
    return 0
}

# Install to Flash Function
install(){
    # Warning Screen!
    dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "WARNING: THIS COULD BRICK YOUR ZIPIT! \
            No one but YOU is responsible if something goes wrong." $DIALOGSIZE
    if [ $? -eq 255 ]; then
        return 255
    fi
    # List the options available to flash
    cd "$flashdir"
    LIST=""
    COUNT=1
    for dir in *; do
            if [ -d "$dir" ]; then
                    export LIST="$LIST $COUNT $dir"
                    export FLASHDIR$COUNT=$dir
                    let COUNT=COUNT+1
            fi
    done

    dialog $BEGIN --backtitle "$BACKTITLE" --nocancel --menu "Choose rootfs to flash:" $DIALOGSIZE 10 $LIST 2>/tmp/flashchoice
    if [ $? -eq 255 ]; then
        return 255
    fi
    flashchoice=`cat /tmp/flashchoice`
    eval flashchoicedir=\$FLASHDIR$flashchoice
    eval flashdefdir="$flashdir/$flashchoicedir"
    cd "$flashdefdir"
    BACKTITLE="$SCRIPTNAME $SCRIPTVER  |  $flashchoicedir"
    
    if ! [ -e "$flashdefs" ]; then
        dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "Flash definition file not found for $flashchoicedir, flash cancelled.." $DIALOGSIZE
        sleep 3
        return 1
    fi

    # View the changelog if found
    if [ -f "$changelog" ]; then
        dialog $BEGIN --backtitle "$BACKTITLE Changelog" --textbox "$changelog" 15 44
    fi

    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Preparing flash image.." $DIALOGSIZE
    if [ -e "$image" ] ; then
            rm -f "$image"
    fi
    touch "$image"
    
    if [ -e "$partlist" ] ; then
            rm -f "$partlist"
    fi
    touch "$partlist"
    
    # create the complete image file with proper spacing
    offset=0
    count=0
    while read file start md5; do
            if ! [ -e "$file" ] ; then
                    dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "File \"$file\" not found, aborting!" $DIALOGSIZE
                    sleep 3
                    return 1
            fi
    
            # save uboot and env md5's for later
            if [ $count -eq 0 ]; then
                    ubootmd5="$md5"
            fi
            if [ $count -eq 1 ]; then
                    envmd5="$md5"
            fi
    
            # check md5sum
            if [ -z "$md5" ] ; then
                    dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "md5sum for $file not found, aborting!" $DIALOGSIZE
                    return 1
            elif [ "$md5" != "`md5sum $file | cut -d " " -f 1`" ] ; then
                    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "md5sum for $file does not match, aborting!" $DIALOGSIZE
                    sleep 3
                    return 1
            fi
    
            # calculate empty space between parts
            pad=`expr "$start" - "$offset"`
    
            if [ "$pad" -lt 0 ] ; then
                    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Image components overlap, aborting!" $DIALOGSIZE
                    sleep 3
                    return 1
            elif [ "$pad" -gt 0 ] ; then
                    # write empty space to image file
                    dd if=/dev/zero bs="$pad" count=1 >> "$image" 2>/dev/null
            fi
    
            # write part to image file
            cat "$file" >> "$image"
            # calculate next offset
            offset=`wc -c "$image" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
            #echo "Offset: $offset"
            md5="\0"
            count=`expr "$count" + 1`
    done < "$flashdefs"
    
    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "$image ready for flashing." $DIALOGSIZE
    
    # fix mtdblock partitions, create partition list and backup old flash
    flashsz=0
    while read line; do
            i=`echo $line | grep mtdblock | cut -d " " -f 4`
            if [ -n "$i" ]; then
                    # fix the mtdblock
                    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Fixing /dev/$i" $DIALOGSIZE
                    blocknum=`echo "$i" | sed -e "s/^.*\(.\)$/\1/"`
                    rm "/dev/$i"
                    mknod "/dev/$i" b 31 "$blocknum"
                    # write the partition list
                    partsz=`wc -c "/dev/$i" | sed -r "s/[[:blank:]]*([[:digit:]]+).*/\1/"`
                    echo "/dev/$i $partsz" >> "$partlist"
  
                    # Check md5 of currently installed u-boot and env
                    if [ $blocknum -eq 0 ]; then
                            thismd5=`md5sum "/dev/$i" | cut -d " " -f 1`
                            if [ "$ubootmd5" != "$thismd5" ]; then
                                    ubootdiff=1
                                    echo "u-boot to flash is different" >>/tmp/uflash
                            else
                                    ubootdiff=0
                                    echo "u-boot to flash is the same" >>/tmp/uflash
                            fi
                    fi
                    if [ $blocknum -eq 1 ]; then
                            thismd5=`md5sum "/dev/$i" | cut -d " " -f 1`
                            if [ "$envmd5" != "$thismd5" ]; then
                                    envdiff=1
                                    echo "env to flash is different" >>/tmp/uflash
                            else
                                    envdiff=0
                                    echo "env to flash is the same" >>/tmp/uflash
                            fi
                    fi
                    flashsz=`expr "$flashsz" + "$partsz"`
                    mtdcount=`expr "$blocknum" + 1`
            fi
    done </proc/partitions
    
    if [ "$offset" -gt "$flashsz" ] ; then
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Image too big for flash, aborting! Offset: $offset FlashSize: $flashsz" $DIALOGSIZE
            sleep 2
            return 1
    fi
    
    if [ -z "$partlist" ] ; then
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox  "Partition list empty, aborting!" $DIALOGSIZE
            sleep 2
            return 1
    fi
    
    # if bootloader is different, recommend to flash
    if [ $ubootdiff -eq 1 ]; then
        dialog $BEGIN --backtitle "$BACKTITLE" --nocancel --menu "Bootloader currently in flash does not match. Would you like to flash Bootloader?" $DIALOGSIZE 10 1 "Yes" 2 "No" 2>/tmp/ubselect
        if [ $? -eq 255 ]; then
            return 255
        fi
        ubchoice=`cat /tmp/ubselect`
        if [ $ubchoice -eq 1 ]; then
            flashuboot=1
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Bootloader WILL be flashed" $DIALOGSIZE
            sleep 3
        else
            flashuboot=0
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Bootloader will NOT be flashed" $DIALOGSIZE
            sleep 3
        fi
    else
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Bootloader is the same, NOT flashing bootloader." $DIALOGSIZE
            flashuboot=0
            sleep 3
    fi
    
    # if uboot environment is different, recommend to flash
    if [ $envdiff -eq 1 ]; then
        dialog $BEGIN --backtitle "$BACKTITLE" --nocancel --menu "Bootloader settings currently in flash do not match. Would you like to flash Bootloader settings?" $DIALOGSIZE 10 1 "Yes" 2 "No" 2>/tmp/ubeselect
        if [ $? -eq 255 ]; then
            return 255
        fi
        ubechoice=`cat /tmp/ubeselect`
        if [ $ubechoice -eq 1 ]; then
            flashenv=1
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Bootloader settings WILL be flashed" $DIALOGSIZE
            sleep 3
        else
            flashenv=0
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Bootloader settings will NOT be flashed" $DIALOGSIZE
            sleep 3
        fi
    else
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Bootloader settings are the same, NOT flashing environment." $DIALOGSIZE
            flashenv=0
            sleep 3
    fi
    
    dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "Make sure your Zipit is fully charged and you have the AC Adapter plugged in. " $DIALOGSIZE
    
    dialog $BEGIN --backtitle "$BACKTITLE" --no-cancel --inputbox "Last chance before flashing. Are you sure? (type: yes OR no)" $DIALOGSIZE 2>/tmp/nogoingback
    
    flash=`cat /tmp/nogoingback`
    if [ "$flash" != "yes" ] && [ "$flash" != "y" ]; then
            dialog $BEGIN --backtitle "$BACKTITLE" --infobox "You didn't enter yes so aborting" $DIALOGSIZE
            return 1
    fi
    
    
    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Starting flashing procedure, do NOT POWER OFF!" $DIALOGSIZE
    sleep 3
    
    offset=0
    while read part size ; do
            blocknum=`echo "$part" | sed -e "s/^.*\(.\)$/\1/"`
            if [ $blocknum -eq 0 ]; then
                    partdesc="bootloader"
            elif [ $blocknum -eq 1 ]; then
                    partdesc="bootloader environment"
            elif [ $blocknum -eq 2 ]; then
                    partdesc="Kernel"
            elif [ $blocknum -eq 3 ]; then
                    partdesc="Filesystem"
            elif [ $blocknum -eq 4 ]; then
                    partdesc="u-boot Splash"
            fi
            if [ $blocknum -eq 0 ] && [ $flashuboot -eq 0 ]; then
                    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Skipping $part ($partdesc)" $DIALOGSIZE
                    sleep 2
            elif [ $blocknum -eq 1 ] && [ $flashenv -eq 0 ]; then
                    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Skipping $part ($partdesc)" $DIALOGSIZE
                    sleep 2
            else
                    #dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Writing to $part ($partdesc)" $DIALOGSIZE
                    #dd if="$image" of="$part" bs=1 skip="$offset" count="$size"
                    (/mnt/mmcblk0p1/bin/pv-u -n "$image" | dd of="$part" bs=1 skip="$offset" count="$size") 2>&1 | dialog $BEGIN --backtitle "$BACKTITLE" --gauge "Writing to $part ($partdesc)" $DIALOGSIZE 0
            fi
            offset=`expr "$offset" + "$size"`
    done < "$partlist"
    
    rm -f "$partlist" "$image"
    
    dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Flashing complete! The Zipit will boot into the new system after a power cycle.\n\nReturning to menu..." $DIALOGSIZE
    return 0
}

# Menu Function
menu(){
    echo "" > /tmp/dselect # Clear the menu item variable
    dialog $BEGIN --backtitle "$BACKTITLE" --nocancel --menu "Choose an option:" $DIALOGSIZE 10 1 "Create a Backup" 2 "Install to Flash" 3 "Help" 4 "Exit to shell" 5 "Powerdown" 2>/tmp/dselect
    choice=`cat /tmp/dselect`
    if [ $choice -eq 5 ]; then
        dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Powering down.." $DIALOGSIZE
        sleep 2
        poweroff
    elif [ $choice -eq 4 ]; then
        dialog $BEGIN --backtitle "$BACKTITLE" --infobox "Exiting to shell.." $DIALOGSIZE
        sleep 2
        clear
	cat /etc/issue
        exit
    elif [ $choice -eq 3 ]; then
        help
    elif [ $choice -eq 2 ]; then
        install
    elif [ $choice -eq 1 ]; then
        backup
    else
        dialog $BEGIN --backtitle "$BACKTITLE" --msgbox "Error! Exiting to shell.." $DIALOGSIZE
        exit 1
    fi
    return 1
}

# Loop the menu
while true; do
    rm /tmp/*
    BACKTITLE="$SCRIPTNAME $SCRIPTVER  |  $SCRIPTSUBNAME"
    menu
done
